@@ -52,8 +52,10 @@
 
 struct GsdA11yKeyboardManagerPrivate
 {
+        guint             start_idle_id;
         int               xkbEventBase;
         GdkDeviceManager *device_manager;
+        guint             device_added_id;
         gboolean          stickykeys_shortcut_val;
         gboolean          slowkeys_shortcut_val;
         GtkWidget        *stickykeys_alert;
@@ -103,8 +105,8 @@ set_devicepresence_handler (GsdA11yKeyboardManager *manager)
                 return;
 
         manager->priv->device_manager = device_manager;
-        g_signal_connect (G_OBJECT (device_manager), "device-added",
-                          G_CALLBACK (device_added_cb), manager);
+        manager->priv->device_added_id = g_signal_connect (G_OBJECT (device_manager), "device-added",
+                                                           G_CALLBACK (device_added_cb), manager);
 }
 
 static gboolean
@@ -533,6 +535,7 @@ ax_slowkeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
         manager->priv->notification = notify_notification_new (title,
                                                                message,
                                                                "preferences-desktop-accessibility");
+        notify_notification_set_app_name (manager->priv->notification, _("Universal Access"));
         notify_notification_set_timeout (manager->priv->notification, NOTIFICATION_TIMEOUT * 1000);
 
         notify_notification_add_action (manager->priv->notification,
@@ -967,17 +970,18 @@ start_a11y_keyboard_idle_cb (GsdA11yKeyboardManager *manager)
  out:
         gnome_settings_profile_end (NULL);
 
+        manager->priv->start_idle_id = 0;
+
         return FALSE;
 }
 
-
 gboolean
 gsd_a11y_keyboard_manager_start (GsdA11yKeyboardManager *manager,
                                  GError                **error)
 {
         gnome_settings_profile_start (NULL);
 
-        g_idle_add ((GSourceFunc) start_a11y_keyboard_idle_cb, manager);
+        manager->priv->start_idle_id = g_idle_add ((GSourceFunc) start_a11y_keyboard_idle_cb, manager);
 
         gnome_settings_profile_end (NULL);
 
@@ -1020,7 +1024,7 @@ gsd_a11y_keyboard_manager_stop (GsdA11yKeyboardManager *manager)
         g_debug ("Stopping a11y_keyboard manager");
 
         if (p->device_manager != NULL) {
-                g_object_unref (p->device_manager);
+                g_signal_handler_disconnect (p->device_manager, p->device_added_id);
                 p->device_manager = NULL;
         }
 
@@ -1062,10 +1066,6 @@ gsd_a11y_keyboard_manager_set_property (GObject        *object,
                                         const GValue   *value,
                                         GParamSpec     *pspec)
 {
-        GsdA11yKeyboardManager *self;
-
-        self = GSD_A11Y_KEYBOARD_MANAGER (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1079,10 +1079,6 @@ gsd_a11y_keyboard_manager_get_property (GObject        *object,
                                         GValue         *value,
                                         GParamSpec     *pspec)
 {
-        GsdA11yKeyboardManager *self;
-
-        self = GSD_A11Y_KEYBOARD_MANAGER (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1096,9 +1092,6 @@ gsd_a11y_keyboard_manager_constructor (GType                  type,
                                        GObjectConstructParam *construct_properties)
 {
         GsdA11yKeyboardManager      *a11y_keyboard_manager;
-        GsdA11yKeyboardManagerClass *klass;
-
-        klass = GSD_A11Y_KEYBOARD_MANAGER_CLASS (g_type_class_peek (GSD_TYPE_A11Y_KEYBOARD_MANAGER));
 
         a11y_keyboard_manager = GSD_A11Y_KEYBOARD_MANAGER (G_OBJECT_CLASS (gsd_a11y_keyboard_manager_parent_class)->constructor (type,
                                                                                                       n_construct_properties,
@@ -1110,10 +1103,6 @@ gsd_a11y_keyboard_manager_constructor (GType                  type,
 static void
 gsd_a11y_keyboard_manager_dispose (GObject *object)
 {
-        GsdA11yKeyboardManager *a11y_keyboard_manager;
-
-        a11y_keyboard_manager = GSD_A11Y_KEYBOARD_MANAGER (object);
-
         G_OBJECT_CLASS (gsd_a11y_keyboard_manager_parent_class)->dispose (object);
 }
 
@@ -1201,6 +1190,9 @@ gsd_a11y_keyboard_manager_finalize (GObject *object)
 
         g_return_if_fail (a11y_keyboard_manager->priv != NULL);
 
+        if (a11y_keyboard_manager->priv->start_idle_id != 0)
+                g_source_remove (a11y_keyboard_manager->priv->start_idle_id);
+
         G_OBJECT_CLASS (gsd_a11y_keyboard_manager_parent_class)->finalize (object);
 }
 
